/* whatclass - a tiny tool to check the class name of a specified window.
   Copyright (C) 2004 kana <nicht AT s8 DOT xrea DOT con>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdarg.h>
#include <stdio.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>


#define NUMBER_OF(array) (sizeof(array) / sizeof((array)[0]))




static const char WC_CLASS_NAME[] = "whatclass";

#define IDHK_QUIT 1
#define IDHK_CHECK 2








/* Misc. functions */

static void
notice(const char* format, ...)
{
	char buf[80 * 25];
	va_list va;

	va_start(va, format);
	vsnprintf(buf, NUMBER_OF(buf), format, va);
	va_end(va);

	buf[NUMBER_OF(buf) - 1] = '\0';

	MessageBox( NULL, buf, WC_CLASS_NAME,
	            MB_OK | MB_SETFOREGROUND | MB_TOPMOST);
}




static BOOL
is_2k_or_later(void)
{
	OSVERSIONINFO os;

	os.dwOSVersionInfoSize = sizeof(os);
	GetVersionEx(&os);

	return 5 <= os.dwMajorVersion;
}








/* Main functions */

static void
check(void)
{
	HWND target;
	HWND root;
	char target_class_name[256];
	char root_class_name[256];

	target = GetForegroundWindow();
	if (target == NULL) {
		notice("No active window");
		return;
	}

	root = GetAncestor(target, GA_ROOT);

	if (!GetClassName( target, target_class_name,
	                   NUMBER_OF(target_class_name) ))
	{
		return;
	}

	if (root == target) {
		notice( "Class name: ``%s''",
		        target_class_name );
	} else {
		if (!GetClassName( root, root_class_name,
		                   NUMBER_OF(root_class_name) ))
		{
			return;
		}

		notice( "Target:\n"
		        "    Class name: ``%s''\n"
		        "\n"
		        "Its ancestor:\n"
		        "    Class name: ``%s''",
		        target_class_name,
		        root_class_name );
	}
}




static LRESULT CALLBACK
window_procedure(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg) {
	default:
		break;


	case WM_HOTKEY:
		switch (wp) {
		default:
			break;

		case IDHK_QUIT:
			DestroyWindow(hwnd);
			break;

		case IDHK_CHECK:
			check();
			break;
		}
		break;


	case WM_CREATE:
		if (!RegisterHotKey( hwnd, IDHK_QUIT,
		                     MOD_CONTROL | MOD_ALT, VK_F10 ))
		{
			notice("Failed RegisterHotKey() for C-A-F10 = QUIT");
			return -1;
		}

		if (!RegisterHotKey( hwnd, IDHK_CHECK,
		                     MOD_CONTROL | MOD_ALT, VK_F9))
		{
			UnregisterHotKey(hwnd, IDHK_QUIT);
			notice("Failed RegisterHotKey() for C-A-F9 = CHECK");
			return -1;
		}
		break;

	case WM_DESTROY:
		UnregisterHotKey(hwnd, IDHK_QUIT);
		UnregisterHotKey(hwnd, IDHK_CHECK);
		PostQuitMessage(0);
		break;
	}

	return DefWindowProc(hwnd, msg, wp, lp);
}




int WINAPI
WinMain( HINSTANCE hinstance, HINSTANCE hprevinstance,
         LPSTR lpcmdline, int ncmdshow )
{
	MSG msg;
	BOOL ret;
	WNDCLASS wc;
	HWND hwnd;

	msg.wParam = 0;

	ZeroMemory(&wc, sizeof(wc));
	wc.lpfnWndProc = window_procedure;
	wc.hInstance = hinstance;
	wc.lpszClassName = WC_CLASS_NAME;
	if (!RegisterClass(&wc)) {
		notice("Failed RegisterClass()");
		goto E_REGISTER_CLASS;
	}

	hwnd = CreateWindowEx( WS_EX_TOOLWINDOW,
	                       WC_CLASS_NAME,
	                       NULL,
	                       0,
	                       0, 0, 0, 0,
	                       (is_2k_or_later() ? HWND_MESSAGE : NULL),
	                       NULL,
	                       hinstance,
	                       NULL );
	if (hwnd == NULL) {
		notice("Failed CreateWindowEx()");
		goto E_CREATE_WINDOW;
	}

	notice( "How to use\n"
	        "==========\n"
	        "\n"
	        "Ctrl-Alt-F9 -- Check\n"
	        "    Check the class name of a plugin.\n"
	        "    You have to click a plugin before checking.\n"
	        "\n"
	        "Ctrl-Alt-F10 -- Quit\n"
	        "    Quit this program.\n" );

	while (TRUE) {
		ret = GetMessage(&msg, NULL, 0, 0);
		if (ret == 0 || ret == -1)
			break;

		DispatchMessage(&msg);
	}

	notice("Good-bye!\n");

E_CREATE_WINDOW:
	UnregisterClass(WC_CLASS_NAME, hinstance);
E_REGISTER_CLASS:
	return msg.wParam;
}




/* __END__ */
